@@ -173,6 +173,10 @@ impl<'db, DB: HirDatabase> Semantics<'db, DB> {
         self.imp.descend_node_at_offset(node, offset).find_map(N::cast)
     }
 
+    pub fn hir_file_for(&self, syntax_node: &SyntaxNode) -> HirFileId {
+        self.imp.find_file(syntax_node.clone()).file_id
+    }
+
     pub fn original_range(&self, node: &SyntaxNode) -> FileRange {
         self.imp.original_range(node)
     }
@@ -199,7 +203,7 @@ impl<'db, DB: HirDatabase> Semantics<'db, DB> {
         self.imp.ancestors_at_offset_with_macros(node, offset)
     }
 
-    /// Find a AstNode by offset inside SyntaxNode, if it is inside *Macrofile*,
+    /// Find an AstNode by offset inside SyntaxNode, if it is inside *Macrofile*,
     /// search up until it is of the target AstNode type
     pub fn find_node_at_offset_with_macros<N: AstNode>(
         &self,
@@ -209,7 +213,7 @@ impl<'db, DB: HirDatabase> Semantics<'db, DB> {
         self.imp.ancestors_at_offset_with_macros(node, offset).find_map(N::cast)
     }
 
-    /// Find a AstNode by offset inside SyntaxNode, if it is inside *MacroCall*,
+    /// Find an AstNode by offset inside SyntaxNode, if it is inside *MacroCall*,
     /// descend it and find again
     pub fn find_node_at_offset_with_descend<N: AstNode>(
         &self,
@@ -274,6 +278,10 @@ impl<'db, DB: HirDatabase> Semantics<'db, DB> {
         self.imp.resolve_macro_call(macro_call)
     }
 
+    pub fn resolve_attr_macro_call(&self, item: &ast::Item) -> Option<MacroDef> {
+        self.imp.resolve_attr_macro_call(item)
+    }
+
     pub fn resolve_path(&self, path: &ast::Path) -> Option<PathResolution> {
         self.imp.resolve_path(path)
     }
@@ -630,6 +638,12 @@ impl<'db> SemanticsImpl<'db> {
         sa.resolve_macro_call(self.db, macro_call)
     }
 
+    fn resolve_attr_macro_call(&self, item: &ast::Item) -> Option<MacroDef> {
+        let item_in_file = self.find_file(item.syntax().clone()).with_value(item.clone());
+        let macro_call_id = self.with_ctx(|ctx| ctx.item_to_macro_call(item_in_file))?;
+        Some(MacroDef { id: self.db.lookup_intern_macro(macro_call_id).def })
+    }
+
     fn resolve_path(&self, path: &ast::Path) -> Option<PathResolution> {
         self.analyze(path.syntax()).resolve_path(self.db, path)
     }
@@ -884,6 +898,7 @@ to_def_impls![
     (crate::Local, ast::IdentPat, bind_pat_to_def),
     (crate::Local, ast::SelfParam, self_param_to_def),
     (crate::Label, ast::Label, label_to_def),
+    (crate::Adt, ast::Adt, adt_to_def),
 ];
 
 fn find_root(node: &SyntaxNode) -> SyntaxNode {
